@@ -11,6 +11,7 @@ import fnmatch
 import functools
 import json
 import os
+import pathlib
 import queue
 import re
 import shutil
@@ -25,10 +26,30 @@ import typing as t
 
 from collections import namedtuple
 from contextlib import contextmanager
+from dataclasses import dataclass, fields as dc_fields
 from hashlib import sha256
 from io import BytesIO
+from importlib.metadata import distribution
 from itertools import chain
 
+try:
+    from packaging.requirements import Requirement as PkgReq
+except ImportError:
+    class PkgReq:  # type: ignore[no-redef]
+        pass
+
+    HAS_PACKAGING = False
+else:
+    HAS_PACKAGING = True
+
+try:
+    from distlib.manifest import Manifest  # type: ignore[import]
+    from distlib import DistlibException  # type: ignore[import]
+except ImportError:
+    HAS_DISTLIB = False
+else:
+    HAS_DISTLIB = True
+
 if t.TYPE_CHECKING:
     from ansible.galaxy.collection.concrete_artifact_manager import (
         ConcreteArtifactsManager,
@@ -63,6 +84,7 @@ if t.TYPE_CHECKING:
     FilesManifestType = t.Dict[t.Literal['files', 'format'], t.Union[t.List[FileManifestEntryType], int]]
 
 import ansible.constants as C
+from ansible.compat.importlib_resources import files
 from ansible.errors import AnsibleError
 from ansible.galaxy.api import GalaxyAPI
 from ansible.galaxy.collection.concrete_artifact_manager import (
@@ -79,23 +101,36 @@ from ansible.galaxy.collection.gpg import (
     get_signature_from_source,
     GPG_ERROR_MAP,
 )
-from ansible.galaxy.dependency_resolution import (
-    build_collection_dependency_resolver,
-)
+try:
+    from ansible.galaxy.dependency_resolution import (
+        build_collection_dependency_resolver,
+    )
+    from ansible.galaxy.dependency_resolution.errors import (
+        CollectionDependencyResolutionImpossible,
+        CollectionDependencyInconsistentCandidate,
+    )
+    from ansible.galaxy.dependency_resolution.providers import (
+        RESOLVELIB_VERSION,
+        RESOLVELIB_LOWERBOUND,
+        RESOLVELIB_UPPERBOUND,
+    )
+except ImportError:
+    HAS_RESOLVELIB = False
+else:
+    HAS_RESOLVELIB = True
+
 from ansible.galaxy.dependency_resolution.dataclasses import (
     Candidate, Requirement, _is_installed_collection_dir,
 )
-from ansible.galaxy.dependency_resolution.errors import (
-    CollectionDependencyResolutionImpossible,
-    CollectionDependencyInconsistentCandidate,
-)
 from ansible.galaxy.dependency_resolution.versioning import meets_requirements
-from ansible.module_utils.six import raise_from
+from ansible.plugins.loader import get_all_plugin_loaders
 from ansible.module_utils._text import to_bytes, to_native, to_text
+from ansible.module_utils.common.collections import is_sequence
 from ansible.module_utils.common.yaml import yaml_dump
 from ansible.utils.collection_loader import AnsibleCollectionRef
 from ansible.utils.display import Display
 from ansible.utils.hashing import secure_hash, secure_hash_s
+from ansible.utils.sentinel import Sentinel
 
 
 display = Display()
@@ -108,6 +143,20 @@ ModifiedContent = namedtuple('ModifiedContent', ['filename', 'expected', 'instal
 SIGNATURE_COUNT_RE = r"^(?P<strict>\+)?(?:(?P<count>\d+)|(?P<all>all))$"
 
 
+@dataclass
+class ManifestControl:
+    directives: list[str] = None
+    omit_default_directives: bool = False
+
+    def __post_init__(self):
+        # Allow a dict representing this dataclass to be splatted directly.
+        # Requires attrs to have a default value, so anything with a default
+        # of None is swapped for its, potentially mutable, default
+        for field in dc_fields(self):
+            if getattr(self, field.name) is None:
+                super().__setattr__(field.name, field.type())
+
+
 class CollectionSignatureError(Exception):
     def __init__(self, reasons=None, stdout=None, rc=None, ignore=False):
         self.reasons = reasons
@@ -155,10 +204,8 @@ class CollectionVerifyResult:
         self.success = True  # type: bool
 
 
-def verify_local_collection(
-        local_collection, remote_collection,
-        artifacts_manager,
-):  # type: (Candidate, Candidate | None, ConcreteArtifactsManager) -> CollectionVerifyResult
+def verify_local_collection(local_collection, remote_collection, artifacts_manager):
+    # type: (Candidate, t.Optional[Candidate], ConcreteArtifactsManager) -> CollectionVerifyResult
     """Verify integrity of the locally installed collection.
 
     :param local_collection: Collection being checked.
@@ -168,9 +215,7 @@ def verify_local_collection(
     """
     result = CollectionVerifyResult(local_collection.fqcn)
 
-    b_collection_path = to_bytes(
-        local_collection.src, errors='surrogate_or_strict',
-    )
+    b_collection_path = to_bytes(local_collection.src, errors='surrogate_or_strict')
 
     display.display("Verifying '{coll!s}'.".format(coll=local_collection))
     display.display(
@@ -426,7 +471,7 @@ def build_collection(u_collection_path, u_output_path, force):
     try:
         collection_meta = _get_meta_from_src_dir(b_collection_path)
     except LookupError as lookup_err:
-        raise_from(AnsibleError(to_native(lookup_err)), lookup_err)
+        raise AnsibleError(to_native(lookup_err)) from lookup_err
 
     collection_manifest = _build_manifest(**collection_meta)
     file_manifest = _build_files_manifest(
@@ -434,6 +479,8 @@ def build_collection(u_collection_path, u_output_path, force):
         collection_meta['namespace'],  # type: ignore[arg-type]
         collection_meta['name'],  # type: ignore[arg-type]
         collection_meta['build_ignore'],  # type: ignore[arg-type]
+        collection_meta['manifest'],  # type: ignore[arg-type]
+        collection_meta['license_file'],  # type: ignore[arg-type]
     )
 
     artifact_tarball_file_name = '{ns!s}-{name!s}-{ver!s}.tar.gz'.format(
@@ -487,6 +534,7 @@ def download_collections(
             upgrade=False,
             # Avoid overhead getting signatures since they are not currently applicable to downloaded collections
             include_signatures=False,
+            offline=False,
         )
 
     b_output_path = to_bytes(output_path, errors='surrogate_or_strict')
@@ -612,6 +660,7 @@ def install_collections(
         allow_pre_release,  # type: bool
         artifacts_manager,  # type: ConcreteArtifactsManager
         disable_gpg_verify,  # type: bool
+        offline,  # type: bool
 ):  # type: (...) -> None
     """Install Ansible collections to the path specified.
 
@@ -689,6 +738,7 @@ def install_collections(
             allow_pre_release=allow_pre_release,
             upgrade=upgrade,
             include_signatures=not disable_gpg_verify,
+            offline=offline,
         )
 
     keyring_exists = artifacts_manager.keyring is not None
@@ -719,6 +769,9 @@ def install_collections(
                     "Skipping signature verification."
                 )
 
+            if concrete_coll_pin.type == 'galaxy':
+                concrete_coll_pin = concrete_coll_pin.with_signatures_repopulated()
+
             try:
                 install(concrete_coll_pin, output_path, artifacts_manager)
             except AnsibleError as err:
@@ -863,7 +916,7 @@ def verify_collections(
                         # NOTE: If there are no Galaxy server signatures, only user-provided signature URLs,
                         # NOTE: those alone validate the MANIFEST.json and the remote collection is not downloaded.
                         # NOTE: The remote MANIFEST.json is only used in verification if there are no signatures.
-                        if not signatures and not collection.signature_sources:
+                        if artifacts_manager.keyring is None or not signatures:
                             api_proxy.get_collection_version_metadata(
                                 remote_collection,
                             )
@@ -881,10 +934,7 @@ def verify_collections(
                             )
                         raise
 
-                result = verify_local_collection(
-                    local_collection, remote_collection,
-                    artifacts_manager,
-                )
+                result = verify_local_collection(local_collection, remote_collection, artifacts_manager)
 
                 results.append(result)
 
@@ -992,7 +1042,153 @@ def _verify_file_hash(b_path, filename, expected_hash, error_queue):
         error_queue.append(ModifiedContent(filename=filename, expected=expected_hash, installed=actual_hash))
 
 
-def _build_files_manifest(b_collection_path, namespace, name, ignore_patterns):
+def _make_manifest():
+    return {
+        'files': [
+            {
+                'name': '.',
+                'ftype': 'dir',
+                'chksum_type': None,
+                'chksum_sha256': None,
+                'format': MANIFEST_FORMAT,
+            },
+        ],
+        'format': MANIFEST_FORMAT,
+    }
+
+
+def _make_entry(name, ftype, chksum_type='sha256', chksum=None):
+    return {
+        'name': name,
+        'ftype': ftype,
+        'chksum_type': chksum_type if chksum else None,
+        f'chksum_{chksum_type}': chksum,
+        'format': MANIFEST_FORMAT
+    }
+
+
+def _build_files_manifest(b_collection_path, namespace, name, ignore_patterns,
+                          manifest_control, license_file):
+    # type: (bytes, str, str, list[str], dict[str, t.Any], t.Optional[str]) -> FilesManifestType
+    if ignore_patterns and manifest_control is not Sentinel:
+        raise AnsibleError('"build_ignore" and "manifest" are mutually exclusive')
+
+    if manifest_control is not Sentinel:
+        return _build_files_manifest_distlib(
+            b_collection_path,
+            namespace,
+            name,
+            manifest_control,
+            license_file,
+        )
+
+    return _build_files_manifest_walk(b_collection_path, namespace, name, ignore_patterns)
+
+
+def _build_files_manifest_distlib(b_collection_path, namespace, name, manifest_control,
+                                  license_file):
+    # type: (bytes, str, str, dict[str, t.Any], t.Optional[str]) -> FilesManifestType
+    if not HAS_DISTLIB:
+        raise AnsibleError('Use of "manifest" requires the python "distlib" library')
+
+    if manifest_control is None:
+        manifest_control = {}
+
+    try:
+        control = ManifestControl(**manifest_control)
+    except TypeError as ex:
+        raise AnsibleError(f'Invalid "manifest" provided: {ex}')
+
+    if not is_sequence(control.directives):
+        raise AnsibleError(f'"manifest.directives" must be a list, got: {control.directives.__class__.__name__}')
+
+    if not isinstance(control.omit_default_directives, bool):
+        raise AnsibleError(
+            '"manifest.omit_default_directives" is expected to be a boolean, got: '
+            f'{control.omit_default_directives.__class__.__name__}'
+        )
+
+    if control.omit_default_directives and not control.directives:
+        raise AnsibleError(
+            '"manifest.omit_default_directives" was set to True, but no directives were defined '
+            'in "manifest.directives". This would produce an empty collection artifact.'
+        )
+
+    directives = []
+    if control.omit_default_directives:
+        directives.extend(control.directives)
+    else:
+        directives.extend([
+            'include meta/*.yml',
+            'include *.txt *.md *.rst *.license COPYING LICENSE',
+            'recursive-include .reuse **',
+            'recursive-include LICENSES **',
+            'recursive-include tests **',
+            'recursive-include docs **.rst **.yml **.yaml **.json **.j2 **.txt **.license',
+            'recursive-include roles **.yml **.yaml **.json **.j2 **.license',
+            'recursive-include playbooks **.yml **.yaml **.json **.license',
+            'recursive-include changelogs **.yml **.yaml **.license',
+            'recursive-include plugins */**.py */**.license',
+        ])
+
+        if license_file:
+            directives.append(f'include {license_file}')
+
+        plugins = set(l.package.split('.')[-1] for d, l in get_all_plugin_loaders())
+        for plugin in sorted(plugins):
+            if plugin in ('modules', 'module_utils'):
+                continue
+            elif plugin in C.DOCUMENTABLE_PLUGINS:
+                directives.append(
+                    f'recursive-include plugins/{plugin} **.yml **.yaml'
+                )
+
+        directives.extend([
+            'recursive-include plugins/modules **.ps1 **.yml **.yaml **.license',
+            'recursive-include plugins/module_utils **.ps1 **.psm1 **.cs **.license',
+        ])
+
+        directives.extend(control.directives)
+
+        directives.extend([
+            f'exclude galaxy.yml galaxy.yaml MANIFEST.json FILES.json {namespace}-{name}-*.tar.gz',
+            'recursive-exclude tests/output **',
+            'global-exclude /.* /__pycache__ *.pyc *.pyo *.bak *~ *.swp',
+        ])
+
+    display.vvv('Manifest Directives:')
+    display.vvv(textwrap.indent('\n'.join(directives), '    '))
+
+    u_collection_path = to_text(b_collection_path, errors='surrogate_or_strict')
+    m = Manifest(u_collection_path)
+    for directive in directives:
+        try:
+            m.process_directive(directive)
+        except DistlibException as e:
+            raise AnsibleError(f'Invalid manifest directive: {e}')
+        except Exception as e:
+            raise AnsibleError(f'Unknown error processing manifest directive: {e}')
+
+    manifest = _make_manifest()
+
+    for abs_path in m.sorted(wantdirs=True):
+        rel_path = os.path.relpath(abs_path, u_collection_path)
+        if os.path.isdir(abs_path):
+            manifest_entry = _make_entry(rel_path, 'dir')
+        else:
+            manifest_entry = _make_entry(
+                rel_path,
+                'file',
+                chksum_type='sha256',
+                chksum=secure_hash(abs_path, hash_func=sha256)
+            )
+
+        manifest['files'].append(manifest_entry)
+
+    return manifest
+
+
+def _build_files_manifest_walk(b_collection_path, namespace, name, ignore_patterns):
     # type: (bytes, str, str, list[str]) -> FilesManifestType
     # We always ignore .pyc and .retry files as well as some well known version control directories. The ignore
     # patterns can be extended by the build_ignore key in galaxy.yml
@@ -1010,25 +1206,7 @@ def _build_files_manifest(b_collection_path, namespace, name, ignore_patterns):
     b_ignore_patterns += [to_bytes(p) for p in ignore_patterns]
     b_ignore_dirs = frozenset([b'CVS', b'.bzr', b'.hg', b'.git', b'.svn', b'__pycache__', b'.tox'])
 
-    entry_template = {
-        'name': None,
-        'ftype': None,
-        'chksum_type': None,
-        'chksum_sha256': None,
-        'format': MANIFEST_FORMAT
-    }
-    manifest = {
-        'files': [
-            {
-                'name': '.',
-                'ftype': 'dir',
-                'chksum_type': None,
-                'chksum_sha256': None,
-                'format': MANIFEST_FORMAT,
-            },
-        ],
-        'format': MANIFEST_FORMAT,
-    }  # type: FilesManifestType
+    manifest = _make_manifest()
 
     def _walk(b_path, b_top_level_dir):
         for b_item in os.listdir(b_path):
@@ -1051,11 +1229,7 @@ def _build_files_manifest(b_collection_path, namespace, name, ignore_patterns):
                                         % to_text(b_abs_path))
                         continue
 
-                manifest_entry = entry_template.copy()
-                manifest_entry['name'] = rel_path
-                manifest_entry['ftype'] = 'dir'
-
-                manifest['files'].append(manifest_entry)
+                manifest['files'].append(_make_entry(rel_path, 'dir'))
 
                 if not os.path.islink(b_abs_path):
                     _walk(b_abs_path, b_top_level_dir)
@@ -1066,13 +1240,14 @@ def _build_files_manifest(b_collection_path, namespace, name, ignore_patterns):
 
                 # Handling of file symlinks occur in _build_collection_tar, the manifest for a symlink is the same for
                 # a normal file.
-                manifest_entry = entry_template.copy()
-                manifest_entry['name'] = rel_path
-                manifest_entry['ftype'] = 'file'
-                manifest_entry['chksum_type'] = 'sha256'
-                manifest_entry['chksum_sha256'] = secure_hash(b_abs_path, hash_func=sha256)
-
-                manifest['files'].append(manifest_entry)
+                manifest['files'].append(
+                    _make_entry(
+                        rel_path,
+                        'file',
+                        chksum_type='sha256',
+                        chksum=secure_hash(b_abs_path, hash_func=sha256)
+                    )
+                )
 
     _walk(b_collection_path, b_collection_path)
 
@@ -1209,54 +1384,97 @@ def _build_collection_dir(b_collection_path, b_collection_output, collection_man
         src_file = os.path.join(b_collection_path, to_bytes(file_info['name'], errors='surrogate_or_strict'))
         dest_file = os.path.join(b_collection_output, to_bytes(file_info['name'], errors='surrogate_or_strict'))
 
-        existing_is_exec = os.stat(src_file).st_mode & stat.S_IXUSR
+        existing_is_exec = os.stat(src_file, follow_symlinks=False).st_mode & stat.S_IXUSR
         mode = 0o0755 if existing_is_exec else 0o0644
 
-        if os.path.isdir(src_file):
+        # ensure symlinks to dirs are not translated to empty dirs
+        if os.path.isdir(src_file) and not os.path.islink(src_file):
             mode = 0o0755
             base_directories.append(src_file)
             os.mkdir(dest_file, mode)
         else:
-            shutil.copyfile(src_file, dest_file)
+            # do not follow symlinks to ensure the original link is used
+            shutil.copyfile(src_file, dest_file, follow_symlinks=False)
+
+        # avoid setting specific permission on symlinks since it does not
+        # support avoid following symlinks and will thrown an exception if the
+        # symlink target does not exist
+        if not os.path.islink(dest_file):
+            os.chmod(dest_file, mode)
 
-        os.chmod(dest_file, mode)
     collection_output = to_text(b_collection_output)
     return collection_output
 
 
-def find_existing_collections(path, artifacts_manager):
+def _normalize_collection_path(path):
+    str_path = path.as_posix() if isinstance(path, pathlib.Path) else path
+    return pathlib.Path(
+        # This is annoying, but GalaxyCLI._resolve_path did it
+        os.path.expandvars(str_path)
+    ).expanduser().absolute()
+
+
+def find_existing_collections(path_filter, artifacts_manager, namespace_filter=None, collection_filter=None, dedupe=True):
     """Locate all collections under a given path.
 
     :param path: Collection dirs layout search path.
     :param artifacts_manager: Artifacts manager.
     """
-    b_path = to_bytes(path, errors='surrogate_or_strict')
+    if files is None:
+        raise AnsibleError('importlib_resources is not installed and is required')
 
-    # FIXME: consider using `glob.glob()` to simplify looping
-    for b_namespace in os.listdir(b_path):
-        b_namespace_path = os.path.join(b_path, b_namespace)
-        if os.path.isfile(b_namespace_path):
+    if path_filter and not is_sequence(path_filter):
+        path_filter = [path_filter]
+
+    paths = set()
+    for path in files('ansible_collections').glob('*/*/'):
+        path = _normalize_collection_path(path)
+        if not path.is_dir():
+            continue
+        if path_filter:
+            for pf in path_filter:
+                try:
+                    path.relative_to(_normalize_collection_path(pf))
+                except ValueError:
+                    continue
+                break
+            else:
+                continue
+        paths.add(path)
+
+    seen = set()
+    for path in paths:
+        namespace = path.parent.name
+        name = path.name
+        if namespace_filter and namespace != namespace_filter:
+            continue
+        if collection_filter and name != collection_filter:
             continue
 
-        # FIXME: consider feeding b_namespace_path to Candidate.from_dir_path to get subdirs automatically
-        for b_collection in os.listdir(b_namespace_path):
-            b_collection_path = os.path.join(b_namespace_path, b_collection)
-            if not os.path.isdir(b_collection_path):
-                continue
-
+        if dedupe:
             try:
-                req = Candidate.from_dir_path_as_unknown(
-                    b_collection_path,
-                    artifacts_manager,
-                )
-            except ValueError as val_err:
-                raise_from(AnsibleError(val_err), val_err)
+                collection_path = files(f'ansible_collections.{namespace}.{name}')
+            except ImportError:
+                continue
+            if collection_path in seen:
+                continue
+            seen.add(collection_path)
+        else:
+            collection_path = path
 
-            display.vvv(
-                u"Found installed collection {coll!s} at '{path!s}'".
-                format(coll=to_text(req), path=to_text(req.src))
-            )
-            yield req
+        b_collection_path = to_bytes(collection_path.as_posix())
+
+        try:
+            req = Candidate.from_dir_path_as_unknown(b_collection_path, artifacts_manager)
+        except ValueError as val_err:
+            display.warning(f'{val_err}')
+            continue
+
+        display.vvv(
+            u"Found installed collection {coll!s} at '{path!s}'".
+            format(coll=to_text(req), path=to_text(req.src))
+        )
+        yield req
 
 
 def install(collection, path, artifacts_manager):  # FIXME: mv to dataclasses?
@@ -1389,11 +1607,7 @@ def install_artifact(b_coll_targz_path, b_collection_path, b_temp_path, signatur
         raise
 
 
-def install_src(
-        collection,
-        b_collection_path, b_collection_output_path,
-        artifacts_manager,
-):
+def install_src(collection, b_collection_path, b_collection_output_path, artifacts_manager):
     r"""Install the collection from source control into given dir.
 
     Generates the Ansible collection artifact data from a galaxy.yml and
@@ -1414,11 +1628,14 @@ def install_src(
     if 'build_ignore' not in collection_meta:  # installed collection, not src
         # FIXME: optimize this? use a different process? copy instead of build?
         collection_meta['build_ignore'] = []
+        collection_meta['manifest'] = Sentinel
     collection_manifest = _build_manifest(**collection_meta)
     file_manifest = _build_files_manifest(
         b_collection_path,
         collection_meta['namespace'], collection_meta['name'],
         collection_meta['build_ignore'],
+        collection_meta['manifest'],
+        collection_meta['license_file'],
     )
 
     collection_output_path = _build_collection_dir(
@@ -1578,8 +1795,33 @@ def _resolve_depenency_map(
         allow_pre_release,  # type: bool
         upgrade,  # type: bool
         include_signatures,  # type: bool
+        offline,  # type: bool
 ):  # type: (...) -> dict[str, Candidate]
     """Return the resolved dependency map."""
+    if not HAS_RESOLVELIB:
+        raise AnsibleError("Failed to import resolvelib, check that a supported version is installed")
+    if not HAS_PACKAGING:
+        raise AnsibleError("Failed to import packaging, check that a supported version is installed")
+
+    req = None
+
+    try:
+        dist = distribution('ansible-core')
+    except Exception:
+        pass
+    else:
+        req = next((rr for r in (dist.requires or []) if (rr := PkgReq(r)).name == 'resolvelib'), None)
+    finally:
+        if req is None:
+            # TODO: replace the hardcoded versions with a warning if the dist info is missing
+            # display.warning("Unable to find 'ansible-core' distribution requirements to verify the resolvelib version is supported.")
+            if not RESOLVELIB_LOWERBOUND <= RESOLVELIB_VERSION < RESOLVELIB_UPPERBOUND:
+                raise AnsibleError(
+                    f"ansible-galaxy requires resolvelib<{RESOLVELIB_UPPERBOUND.vstring},>={RESOLVELIB_LOWERBOUND.vstring}"
+                )
+        elif not req.specifier.contains(RESOLVELIB_VERSION.vstring):
+            raise AnsibleError(f"ansible-galaxy requires {req.name}{req.specifier}")
+
     collection_dep_resolver = build_collection_dependency_resolver(
         galaxy_apis=galaxy_apis,
         concrete_artifacts_manager=concrete_artifacts_manager,
@@ -1589,6 +1831,7 @@ def _resolve_depenency_map(
         with_pre_releases=allow_pre_release,
         upgrade=upgrade,
         include_signatures=include_signatures,
+        offline=offline,
     )
     try:
         return collection_dep_resolver.resolve(
@@ -1614,10 +1857,7 @@ def _resolve_depenency_map(
             ),
             conflict_causes,
         ))
-        raise raise_from(  # NOTE: Leading "raise" is a hack for mypy bug #9717
-            AnsibleError('\n'.join(error_msg_lines)),
-            dep_exc,
-        )
+        raise AnsibleError('\n'.join(error_msg_lines)) from dep_exc
     except CollectionDependencyInconsistentCandidate as dep_exc:
         parents = [
             "%s.%s:%s" % (p.namespace, p.name, p.ver)
@@ -1644,9 +1884,6 @@ def _resolve_depenency_map(
                 '* {req.fqcn!s}:{req.ver!s}'.format(req=req)
             )
 
-        raise raise_from(  # NOTE: Leading "raise" is a hack for mypy bug #9717
-            AnsibleError('\n'.join(error_msg_lines)),
-            dep_exc,
-        )
+        raise AnsibleError('\n'.join(error_msg_lines)) from dep_exc
     except ValueError as exc:
         raise AnsibleError(to_native(exc)) from exc
