@@ -7,7 +7,7 @@ from __future__ import absolute_import, division, print_function
 __metaclass__ = type
 
 
-DOCUMENTATION = '''
+DOCUMENTATION = r'''
 ---
 module: getent
 short_description: A wrapper to the unix getent utility
@@ -27,7 +27,6 @@ options:
             - Key from which to return values from the specified database, otherwise the
               full contents are returned.
         type: str
-        default: ''
     service:
         description:
             - Override all databases with the specified service
@@ -36,11 +35,12 @@ options:
         version_added: "2.9"
     split:
         description:
-            - "Character used to split the database values into lists/arrays such as ':' or '\t', otherwise  it will try to pick one depending on the database."
+            - Character used to split the database values into lists/arrays such as C(:) or C(\t),
+              otherwise it will try to pick one depending on the database.
         type: str
     fail_key:
         description:
-            - If a supplied key is missing this will make the task fail if C(yes).
+            - If a supplied key is missing this will make the task fail if C(true).
         type: bool
         default: 'yes'
 extends_documentation_fragment:
