@@ -94,9 +94,16 @@ options:
   force_basic_auth:
     description:
       - Force the sending of the Basic authentication header upon initial request.
-      - The library used by the uri module only sends authentication information when a webservice
-        responds to an initial request with a 401 status. Since some basic auth services do not properly
-        send a 401, logins will fail.
+      - When this setting is C(false), this module will first try an unauthenticated request, and when the server replies
+        with an C(HTTP 401) error, it will submit the Basic authentication header.
+      - When this setting is C(true), this module will immediately send a Basic authentication header on the first
+        request.
+      - "Use this setting in any of the following scenarios:"
+      - You know the webservice endpoint always requires HTTP Basic authentication, and you want to speed up your
+        requests by eliminating the first roundtrip.
+      - The web service does not properly send an HTTP 401 error to your client, so Ansible's HTTP library will not
+        properly respond with HTTP credentials, and logins will fail.
+      - The webservice bans or rate-limits clients that cause any HTTP 401 errors.
     type: bool
     default: no
   follow_redirects:
@@ -135,6 +142,7 @@ options:
           of C(2.3) supplying C(Content-Type) here will override the header
           generated by supplying C(json) or C(form-urlencoded) for I(body_format).
     type: dict
+    default: {}
     version_added: '2.1'
   validate_certs:
     description:
@@ -142,7 +150,7 @@ options:
       - This should only set to C(false) used on personally controlled sites using self-signed certificates.
       - Prior to 1.9.2 the code defaulted to C(false).
     type: bool
-    default: yes
+    default: true
     version_added: '1.9.2'
   client_cert:
     description:
@@ -184,7 +192,7 @@ options:
     description:
       - If C(false), it will not use a proxy, even if one is defined in an environment variable on the target hosts.
     type: bool
-    default: yes
+    default: true
   unix_socket:
     description:
     - Path to Unix domain socket to use for connection
@@ -249,12 +257,12 @@ EXAMPLES = r'''
   ansible.builtin.uri:
     url: http://www.example.com
 
-- name: Check that a page returns a status 200 and fail if the word AWESOME is not in the page contents
+- name: Check that a page returns successfully but fail if the word AWESOME is not in the page contents
   ansible.builtin.uri:
     url: http://www.example.com
-    return_content: yes
+    return_content: true
   register: this
-  failed_when: "'AWESOME' not in this.content"
+  failed_when: this is failed or "'AWESOME' not in this.content"
 
 - name: Create a JIRA issue
   ansible.builtin.uri:
@@ -263,7 +271,7 @@ EXAMPLES = r'''
     password: your_pass
     method: POST
     body: "{{ lookup('ansible.builtin.file','issue.json') }}"
-    force_basic_auth: yes
+    force_basic_auth: true
     status_code: 201
     body_format: json
 
@@ -310,7 +318,7 @@ EXAMPLES = r'''
   ansible.builtin.uri:
     url: https://your.form.based.auth.example.com/dashboard.php
     method: GET
-    return_content: yes
+    return_content: true
     headers:
       Cookie: "{{ login.cookies_string }}"
 
@@ -320,7 +328,7 @@ EXAMPLES = r'''
     user: "{{ jenkins.user }}"
     password: "{{ jenkins.password }}"
     method: GET
-    force_basic_auth: yes
+    force_basic_auth: true
     status_code: 201
 
 - name: POST from contents of local file
@@ -334,7 +342,7 @@ EXAMPLES = r'''
     url: https://httpbin.org/post
     method: POST
     src: /path/to/my/file.json
-    remote_src: yes
+    remote_src: true
 
 - name: Create workspaces in Log analytics Azure
   ansible.builtin.uri:
@@ -444,10 +452,10 @@ from ansible.module_utils.basic import AnsibleModule, sanitize_keys
 from ansible.module_utils.six import PY2, PY3, binary_type, iteritems, string_types
 from ansible.module_utils.six.moves.urllib.parse import urlencode, urlsplit
 from ansible.module_utils._text import to_native, to_text
-from ansible.module_utils.common._collections_compat import Mapping, Sequence
+from ansible.module_utils.six.moves.collections_abc import Mapping, Sequence
 from ansible.module_utils.urls import fetch_url, get_response_filename, parse_content_type, prepare_multipart, url_argument_spec
 
-JSON_CANDIDATES = ('text', 'json', 'javascript')
+JSON_CANDIDATES = {'json', 'javascript'}
 
 # List of response key names we do not want sanitize_keys() to change.
 NO_MODIFY_KEYS = frozenset(
@@ -695,11 +703,11 @@ def main():
         content_type, main_type, sub_type, content_encoding = parse_content_type(r)
     else:
         content_type = 'application/octet-stream'
-        main_type = 'aplication'
+        main_type = 'application'
         sub_type = 'octet-stream'
         content_encoding = 'utf-8'
 
-    maybe_json = content_type and any(candidate in sub_type for candidate in JSON_CANDIDATES)
+    maybe_json = content_type and sub_type.lower() in JSON_CANDIDATES
     maybe_output = maybe_json or return_content or info['status'] not in status_code
 
     if maybe_output:
