@@ -48,6 +48,7 @@ from .data import (
 from .host_configs import (
     PosixConfig,
     PythonConfig,
+    VirtualPythonConfig,
 )
 
 from .connections import (
@@ -70,13 +71,14 @@ VIRTUALENV_VERSION = '16.7.12'
 
 class PipUnavailableError(ApplicationError):
     """Exception raised when pip is not available."""
+
     def __init__(self, python: PythonConfig) -> None:
         super().__init__(f'Python {python.version} at "{python.path}" does not have pip available.')
 
 
 @dataclasses.dataclass(frozen=True)
 class PipCommand:
-    """Base class for pip commands."""""
+    """Base class for pip commands."""
 
     def serialize(self) -> tuple[str, dict[str, t.Any]]:
         """Return a serialized representation of this command."""
@@ -87,6 +89,7 @@ class PipCommand:
 @dataclasses.dataclass(frozen=True)
 class PipInstall(PipCommand):
     """Details required to perform a pip install."""
+
     requirements: list[tuple[str, str]]
     constraints: list[tuple[str, str]]
     packages: list[str]
@@ -102,6 +105,7 @@ class PipInstall(PipCommand):
 @dataclasses.dataclass(frozen=True)
 class PipUninstall(PipCommand):
     """Details required to perform a pip uninstall."""
+
     packages: list[str]
     ignore_errors: bool
 
@@ -114,6 +118,7 @@ class PipVersion(PipCommand):
 @dataclasses.dataclass(frozen=True)
 class PipBootstrap(PipCommand):
     """Details required to bootstrap pip."""
+
     pip_version: str
     packages: list[str]
 
@@ -261,6 +266,20 @@ def run_pip(
     connection = connection or LocalConnection(args)
     script = prepare_pip_script(commands)
 
+    if isinstance(args, IntegrationConfig):
+        # Integration tests can involve two hosts (controller and target).
+        # The connection type can be used to disambiguate between the two.
+        context = " (controller)" if isinstance(connection, LocalConnection) else " (target)"
+    else:
+        context = ""
+
+    if isinstance(python, VirtualPythonConfig):
+        context += " [venv]"
+
+    # The interpreter path is not included below.
+    # It can be seen by running ansible-test with increased verbosity (showing all commands executed).
+    display.info(f'Installing requirements for Python {python.version}{context}')
+
     if not args.explain:
         try:
             connection.run([python.path], data=script, capture=False)
